/* packet-dcerpc-AFS4Int.c
 *
 * Routines for dcerpc Afs4Int dissection
 * Copyright 2002, Jaime Fournier <jafour1@yahoo.com> 
 * This information is based off the released idl files from opengroup.
 * ftp://ftp.opengroup.org/pub/dce122/dce/src/file.tar.gz file/fsint/afs4int.idl
 *
 * $Id: packet-dcerpc-afs4int.c,v 1.35 2002/10/27 23:49:47 root Exp root $
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@ethereal.com>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif


#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#include <string.h>

#include <glib.h>
#include <epan/packet.h>
#include "packet-dcerpc.h"
#include "packet-dcerpc-dce122.h"
#include "packet-dcerpc-macros.h"

#define AFS_SETMODTIME  1
#define AFS_SETOWNER  2
#define AFS_SETGROUP  4
#define AFS_SETMODE  8
#define AFS_SETACCESSTIME  0x10
#define AFS_SETCHANGETIME  0x20
#define AFS_SETLENGTH  0x40
#define AFS_SETTYPEUUID  0x80
#define AFS_SETDEVNUM  0x100
#define AFS_SETMODEXACT  0x200
#define AFS_SETTRUNCLENGTH  0x400
#define AFS_SETCLIENTSPARE  0x800

#define	AFS_FLAG_SEC_SERVICE             0x1
#define AFS_FLAG_CONTEXT_NEW_IF          0x2
#define AFS_FLAG_CONTEXT_DO_RESET        0x4
#define AFS_FLAG_CONTEXT_NEW_ACL_IF      0x8
#define AFS_FLAG_CONTEXT_NEW_TKN_TYPES  0x10

#define AFS_FLAG_RETURNTOKEN           1
#define AFS_FLAG_TOKENJUMPQUEUE        2
#define AFS_FLAG_SKIPTOKEN             4
#define AFS_FLAG_NOOPTIMISM           0x8
#define AFS_FLAG_TOKENID              0x10
#define AFS_FLAG_RETURNBLOCKER        0x20
#define AFS_FLAG_ASYNCGRANT           0x40
#define AFS_FLAG_NOREVOKE             0x80
#define AFS_FLAG_MOVE_REESTABLISH     0x100
#define AFS_FLAG_SERVER_REESTABLISH   0x200
#define AFS_FLAG_NO_NEW_EPOCH         0x400
#define AFS_FLAG_MOVE_SOURCE_OK       0x800
#define AFS_FLAG_SYNC                 0x1000
#define AFS_FLAG_ZERO                 0x2000
#define AFS_FLAG_SKIPSTATUS           0x4000
#define AFS_FLAG_FORCEREVOCATIONS     0x8000
#define AFS_FLAG_FORCEVOLQUIESCE      0x10000
#define AFS_FLAG_FORCEREVOCATIONDOWN  0x20000

#define TKN_LOCK_READ                   0x001
#define TKN_LOCK_WRITE                  0x002
#define TKN_DATA_READ                   0x004
#define TKN_DATA_WRITE                  0x008
#define TKN_OPEN_READ                   0x010
#define TKN_OPEN_WRITE                  0x020
#define TKN_OPEN_SHARED                 0x040
#define TKN_OPEN_EXCLUSIVE              0x080
#define TKN_OPEN_DELETE                 0x100
#define TKN_OPEN_PRESERVE               0x200
#define TKN_STATUS_READ                 0x400
#define TKN_STATUS_WRITE                0x800
#define TKN_OPEN_UNLINK                 0x1000
#define TKN_SPOT_HERE                   0x2000
#define TKN_SPOT_THERE                  0x4000
#define TKN_OPEN_NO_READ                0x8000
#define TKN_OPEN_NO_WRITE               0x10000
#define TKN_OPEN_NO_UNLINK              0x20000


/* afsfid vars */
static int hf_afs4int_cell_high = -1;
static int hf_afs4int_cell_low = -1;
static int hf_afs4int_volume_high = -1;
static int hf_afs4int_volume_low = -1;
static int hf_afs4int_vnode = -1;
static int hf_afs4int_unique = -1;
/* afsfetchstatus vars */
static int hf_afs4int_interfaceversion = -1;
static int hf_afs4int_filetype = -1;
static int hf_afs4int_linkcount = -1;
static int hf_afs4int_length_high = -1;
static int hf_afs4int_length_low = -1;
static int hf_afs4int_dataversion_high = -1;
static int hf_afs4int_dataversion_low = -1;
static int hf_afs4int_author = -1;
static int hf_afs4int_owner = -1;
static int hf_afs4int_group = -1;
static int hf_afs4int_calleraccess = -1;
static int hf_afs4int_anonymousaccess = -1;
static int hf_afs4int_aclexpirationtime = -1;
static int hf_afs4int_mode = -1;
static int hf_afs4int_parentvnode = -1;
static int hf_afs4int_parentunique = -1;
static int hf_afs4int_modtime_sec = -1;
static int hf_afs4int_modtime_msec = -1;
static int hf_afs4int_changetime_sec = -1;
static int hf_afs4int_changetime_msec = -1;
static int hf_afs4int_accesstime_sec = -1;
static int hf_afs4int_accesstime_msec = -1;
static int hf_afs4int_servermodtime_sec = -1;
static int hf_afs4int_servermodtime_msec = -1;
static int hf_afs4int_typeuuid = -1;
static int hf_afs4int_objectuuid = -1;
static int hf_afs4int_devicenumber = -1;
static int hf_afs4int_blocksused = -1;
static int hf_afs4int_clientspare1 = -1;
static int hf_afs4int_devicenumberhighbits = -1;
static int hf_afs4int_agtypeunique = -1;
static int hf_afs4int_himaxspare = -1;
static int hf_afs4int_lomaxspare = -1;
static int hf_afs4int_pathconfspare = -1;
static int hf_afs4int_spare4 = -1;
static int hf_afs4int_spare5 = -1;
static int hf_afs4int_spare6 = -1;
/* afsVolSync vars */
static int hf_afs4int_volid_hi = -1;
static int hf_afs4int_volid_low = -1;
static int hf_afs4int_vvage = -1;
static int hf_afs4int_vv_hi = -1;
static int hf_afs4int_vv_low = -1;
static int hf_afs4int_vvpingage = -1;
static int hf_afs4int_vvspare1 = -1;
static int hf_afs4int_vvspare2 = -1;
/* afsToken vars */
static int hf_afs4int_beginrange = -1;
static int hf_afs4int_beginrangeext = -1;
static int hf_afs4int_endrange = -1;
static int hf_afs4int_endrangeext = -1;
static int hf_afs4int_expirationtime = -1;
static int hf_afs4int_tokenid_hi = -1;
static int hf_afs4int_tokenid_low = -1;
static int hf_afs4int_type_hi = -1;
static int hf_afs4int_type_low = -1;
/* afsTaggedName */
static int hf_afs4int_tn_length = -1;
static int hf_afs4int_tn_size = -1;
static int hf_afs4int_tn_string = -1;
static int hf_afs4int_tn_tag = -1;
/* afsStoreStatus vars */
static int hf_afs4int_storestatus_accesstime_sec = -1;
static int hf_afs4int_storestatus_accesstime_usec = -1;
static int hf_afs4int_storestatus_changetime_sec = -1;
static int hf_afs4int_storestatus_changetime_usec = -1;
static int hf_afs4int_storestatus_clientspare1 = -1;
static int hf_afs4int_storestatus_cmask = -1;
static int hf_afs4int_storestatus_devicenumber = -1;
static int hf_afs4int_storestatus_devicenumberhighbits = -1;
static int hf_afs4int_storestatus_devicetype = -1;
static int hf_afs4int_storestatus_group = -1;
static int hf_afs4int_storestatus_length_high = -1;
static int hf_afs4int_storestatus_length_low = -1;
static int hf_afs4int_storestatus_mask = -1;
static int hf_afs4int_storestatus_mode = -1;
static int hf_afs4int_storestatus_modtime_sec = -1;
static int hf_afs4int_storestatus_modtime_usec = -1;
static int hf_afs4int_storestatus_owner = -1;
static int hf_afs4int_storestatus_spare1 = -1;
static int hf_afs4int_storestatus_spare2 = -1;
static int hf_afs4int_storestatus_spare3 = -1;
static int hf_afs4int_storestatus_spare4 = -1;
static int hf_afs4int_storestatus_spare5 = -1;
static int hf_afs4int_storestatus_spare6 = -1;
static int hf_afs4int_storestatus_trunc_high = -1;
static int hf_afs4int_storestatus_trunc_low = -1;
static int hf_afs4int_storestatus_typeuuid = -1;
/* RecordLock vars */
static int hf_afs4int_l_end_pos = -1;
static int hf_afs4int_l_end_pos_ext = -1;
static int hf_afs4int_l_fstype = -1;
static int hf_afs4int_l_pid = -1;
static int hf_afs4int_l_start_pos = -1;
static int hf_afs4int_l_start_pos_ext = -1;
static int hf_afs4int_l_sysid = -1;
static int hf_afs4int_l_type = -1;
static int hf_afs4int_l_whence = -1;

/* general vars */
static int hf_afs4int_minvvp_high = -1;
static int hf_afs4int_minvvp_low = -1;
static int hf_afs4int_flags = -1;
static int hf_afs4int_acl_len = -1;
static int hf_afs4int_acltype = -1;
static int hf_afs4int_st = -1;
static int hf_afs4int_fstype = -1;
static int hf_afs4int_hyper_high = -1;
static int hf_afs4int_hyper_low = -1;
static int hf_afs4int_uint = -1;
static int hf_afs4int_int = -1;

/* old vars */
static int hf_afs4int_setcontext_resp_st = -1;
static int hf_afs4int_setcontext_rqst_epochtime = -1;
static int hf_afs4int_setcontext_rqst_afsnetaddr_type = -1;
static int hf_afs4int_setcontext_rqst_key_t = -1;
static int hf_afs4int_setcontext_rqst_key_size = -1;
static int hf_afs4int_setcontext_rqst_secobjectid = -1;
static int hf_afs4int_setcontext_rqst_clientsizesattrs = -1;
static int hf_afs4int_setcontext_rqst_parm7 = -1;
static int hf_afs4int_setcontext_rqst_data0 = -1;
static int hf_afs4int_setcontext_rqst_data1 = -1;
static int hf_afs4int_setcontext_rqst_data2 = -1;
static int hf_afs4int_setcontext_rqst_data3 = -1;
static int hf_afs4int_setcontext_rqst_data4 = -1;
static int hf_afs4int_setcontext_rqst_data5 = -1;
static int hf_afs4int_setcontext_rqst_data6 = -1;
static int hf_afs4int_setcontext_rqst_data7 = -1;
static int hf_afs4int_setcontext_rqst_data8 = -1;
static int hf_afs4int_setcontext_rqst_data9 = -1;
static int hf_afs4int_setcontext_rqst_data10 = -1;
static int hf_afs4int_setcontext_rqst_data11 = -1;
static int hf_afs4int_setcontext_rqst_data12 = -1;
static int hf_afs4int_setcontext_rqst_data13 = -1;
static int hf_afs4int_setcontext_rqst_data14 = -1;
extern guint32 master_seqnum;

static int proto_AFS4Int = -1;

static gint ett_AFS4Int = -1;


static e_uuid_t uuid_AFS4Int =
  { 0x4d37f2dd, 0xed93, 0x0000, {0x02, 0xc0, 0x37, 0xcf, 0x1e, 0x00, 0x00,
				 0x00}
};
static guint16 ver_AFS4Int = 4;

static int
afs4int_dissect_removefile_rqst (tvbuff_t * tvb, int offset,
				 packet_info * pinfo, proto_tree * tree,
				 char *drep)
{
/*
        [in]    afsFid          *DirFidp,
        [in]    afsFidTaggedName        *Namep,
        [in]    afsHyper        *returnTokenIDp,
        [in]    afsHyper        *minVVp,
        [in]    unsigned32      Flags,
*/

  OUT_AFSFid ();
  OUT_AFSFidTaggedName ();
  OUT_AFSHyper ("returnTokenIDp");
  OUT_AFSHyper ("minVVp");
  OUT_AFSUint ("flags");
  OUT_SEQNUM ();

  return offset;
}

static int
afs4int_dissect_storedata_rqst (tvbuff_t * tvb, int offset,
				packet_info * pinfo, proto_tree * tree,
				char *drep)
{
/*
        [in]    afsFid          *Fidp,
        [in]    afsStoreStatus  *InStatusp,
        [in]    afsHyper        *Position,
        [in]    signed32        Length,
        [in]    afsHyper        *minVVp,
        [in]    unsigned32      Flags,
        [in]    pipe_t          *storeStream,
*/


  OUT_AFSFid ();
  OUT_AFSStoreStatus ();
  OUT_AFSHyper ("Position");
  OUT_AFSUint ("Length");
  OUT_AFSHyper ("minVVp");
  OUT_AFSUint ("Flags");
/* XXX need to decode pipe_t still here */
  OUT_SEQNUM ();

  return offset;
}

static int
afs4int_dissect_gettoken_rqst (tvbuff_t * tvb, int offset,
			       packet_info * pinfo, proto_tree * tree,
			       char *drep)
{

/*
        [in]    afsFid          *Fidp,
        [in]    afsToken        *MinTokenp,
        [in]    afsHyper        *minVVp,
        [in]    unsigned32      Flags,
*/

  OUT_AFSFid ();
  OUT_AFSToken ();
  OUT_AFSHyper ("minVVp");
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_flags, &flags); 

/* Code to check all the flags on flags, and add the proper line  */
  if (check_col (pinfo->cinfo, COL_INFO))
    col_append_str (pinfo->cinfo, COL_INFO, " Flags=");

  if ((flags & AFS_FLAG_RETURNTOKEN) == AFS_FLAG_RETURNTOKEN)
    {
      if (check_col (pinfo->cinfo, COL_INFO))
	col_append_str (pinfo->cinfo, COL_INFO, ":ReturnToken");
    }
  if ((flags & AFS_FLAG_TOKENJUMPQUEUE) == AFS_FLAG_TOKENJUMPQUEUE)
    {
      if (check_col (pinfo->cinfo, COL_INFO))
	col_append_str (pinfo->cinfo, COL_INFO, ":TokenJumpQueue");
    }
  if ((flags & AFS_FLAG_SKIPTOKEN) == AFS_FLAG_SKIPTOKEN)
    {
      if (check_col (pinfo->cinfo, COL_INFO))
	col_append_str (pinfo->cinfo, COL_INFO, ":SkipToken");
    }
  if ((flags & AFS_FLAG_NOOPTIMISM) == AFS_FLAG_NOOPTIMISM)
    {
      if (check_col (pinfo->cinfo, COL_INFO))
	col_append_str (pinfo->cinfo, COL_INFO, ":NoOptimism");
    }
  if ((flags & AFS_FLAG_TOKENID) == AFS_FLAG_TOKENID)
    {
      if (check_col (pinfo->cinfo, COL_INFO))
	col_append_str (pinfo->cinfo, COL_INFO, ":TokenID");
    }
  if ((flags & AFS_FLAG_RETURNBLOCKER) == AFS_FLAG_RETURNBLOCKER)
    {
      if (check_col (pinfo->cinfo, COL_INFO))
	col_append_str (pinfo->cinfo, COL_INFO, ":ReturnBlocker");
    }
  if ((flags & AFS_FLAG_ASYNCGRANT) == AFS_FLAG_ASYNCGRANT)
    {
      if (check_col (pinfo->cinfo, COL_INFO))
	col_append_str (pinfo->cinfo, COL_INFO, ":AsynGrant");
    }
  if ((flags & AFS_FLAG_NOREVOKE) == AFS_FLAG_NOREVOKE)
    {
      if (check_col (pinfo->cinfo, COL_INFO))
	col_append_str (pinfo->cinfo, COL_INFO, ":NoRevoke");
    }
  if ((flags & AFS_FLAG_MOVE_REESTABLISH) == AFS_FLAG_MOVE_REESTABLISH)
    {
      if (check_col (pinfo->cinfo, COL_INFO))
	col_append_str (pinfo->cinfo, COL_INFO, ":MoveReestablish");
    }
  if ((flags & AFS_FLAG_SERVER_REESTABLISH) == AFS_FLAG_SERVER_REESTABLISH)
    {
      if (check_col (pinfo->cinfo, COL_INFO))
	col_append_str (pinfo->cinfo, COL_INFO, ":ServerReestablish");
    }
  if ((flags & AFS_FLAG_NO_NEW_EPOCH) == AFS_FLAG_NO_NEW_EPOCH)
    {
      if (check_col (pinfo->cinfo, COL_INFO))
	col_append_str (pinfo->cinfo, COL_INFO, ":NewEpoch");
    }
  if ((flags & AFS_FLAG_MOVE_SOURCE_OK) == AFS_FLAG_MOVE_SOURCE_OK)
    {
      if (check_col (pinfo->cinfo, COL_INFO))
	col_append_str (pinfo->cinfo, COL_INFO, ":MoveSourceOk");
    }
  if ((flags & AFS_FLAG_SYNC) == AFS_FLAG_SYNC)
    {
      if (check_col (pinfo->cinfo, COL_INFO))
	col_append_str (pinfo->cinfo, COL_INFO, ":Sync");
    }
  if ((flags & AFS_FLAG_ZERO) == AFS_FLAG_ZERO)
    {
      if (check_col (pinfo->cinfo, COL_INFO))
	col_append_str (pinfo->cinfo, COL_INFO, ":Zero");
    }
  if ((flags & AFS_FLAG_SKIPSTATUS) == AFS_FLAG_SKIPSTATUS)
    {
      if (check_col (pinfo->cinfo, COL_INFO))
	col_append_str (pinfo->cinfo, COL_INFO, ":SkipStatus");
    }
  if ((flags & AFS_FLAG_FORCEREVOCATIONS) == AFS_FLAG_FORCEREVOCATIONS)
    {
      if (check_col (pinfo->cinfo, COL_INFO))
	col_append_str (pinfo->cinfo, COL_INFO, ":ForceRevocation");
    }
  if ((flags & AFS_FLAG_FORCEVOLQUIESCE) == AFS_FLAG_FORCEVOLQUIESCE)
    {
      if (check_col (pinfo->cinfo, COL_INFO))
	col_append_str (pinfo->cinfo, COL_INFO, ":ForceVolQuiesce");
    }
  if ((flags & AFS_FLAG_FORCEREVOCATIONDOWN) == AFS_FLAG_FORCEREVOCATIONDOWN)
    {
      if (check_col (pinfo->cinfo, COL_INFO))
	col_append_str (pinfo->cinfo, COL_INFO, ":ForceRevocationDown");
    }
  OUT_SEQNUM ();


  return offset;
}
static int
afs4int_dissect_gettoken_resp (tvbuff_t * tvb, int offset,
			       packet_info * pinfo, proto_tree * tree,
			       char *drep)
{

/*
        [out]   afsToken        *OutTokenp,
        [out]   afsRecordLock   *OutBlockerp,
        [out]   afsFetchStatus  *OutStatusp,
        [out]   afsVolSync      *Syncp
*/

  OUT_AFSToken ();
  OUT_AFSRecordLock ();
  OUT_AFSFetchStatus ();
  OUT_AFSVolSync ();
  OUT_ST ();
  OUT_SEQNUM ();

  return offset;
}

static int
afs4int_dissect_lookuproot_rqst (tvbuff_t * tvb, int offset,
				 packet_info * pinfo, proto_tree * tree,
				 char *drep)
{

  /*
   * [in]    afsFid          *InFidp,
   * *        [in]    afsHyper        *minVVp,
   * *        [in]    unsigned32   Flags,
   */

  OUT_AFSFid ();
  OUT_AFSHyper ("minvvp");
  OUT_AFSUint ("flags");
  OUT_SEQNUM ();


  return offset;
}

static int
afs4int_dissect_fetchdata_rqst (tvbuff_t * tvb, int offset,
				packet_info * pinfo, proto_tree * tree,
				char *drep)
{
/*
        [in]    afsFid          *Fidp,
        [in]    afsHyper        *minVVp,
        [in]    afsHyper        *Position,
        [in]    signed32                Length,
        [in]    unsigned32      Flags,
*/

  OUT_AFSFid ();
  OUT_SEQNUM ();


  return offset;
}

static int
afs4int_dissect_fetchacl_rqst (tvbuff_t * tvb, int offset,
			       packet_info * pinfo, proto_tree * tree,
			       char *drep)
{

/*
        [in]    afsFid          *Fidp,
        [in]    unsigned32      aclType,
        [in]    afsHyper        *minVVp,
        [in]    unsigned32      Flags,
*/

  OUT_AFSFid ();
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_acltype, &acltype);
  OUT_AFSHyper ("minvvp");
  OUT_AFSUint ("flags");

  if (acltype)
    {
      if (check_col (pinfo->cinfo, COL_INFO))
	col_append_str (pinfo->cinfo, COL_INFO,
			" :copy the ACL from specified fid:");
    }

  OUT_SEQNUM ();

  return offset;
}
static int
afs4int_dissect_fetchstatus_rqst (tvbuff_t * tvb, int offset,
				  packet_info * pinfo, proto_tree * tree,
				  char *drep)
{
/*
        [in]    afsFid          *Fidp,
        [in]    afsHyper        *minVVp,
        [in]    unsigned32      Flags,
*/

  OUT_AFSFid ();
  OUT_AFSHyper ("minVVp");
  OUT_AFSUint ("flags");
  OUT_SEQNUM ();

  return offset;
}
static int
afs4int_dissect_storeacl_rqst (tvbuff_t * tvb, int offset,
			       packet_info * pinfo, proto_tree * tree,
			       char *drep)
{
/*
        [in]    afsFid          *Fidp,
        [in]    afsACL          *AccessListp,
        [in]    unsigned32      aclType,
        [in]    afsFid          *aclFidp,
        [in]    afsHyper        *minVVp,
        [in]    unsigned32      Flags,
*/


  OUT_AFSFid ();
  OUT_AFSUint2 ("acl_len");
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_acl_len, &acl_len);
  offset += acl_len;		/* XXX need to read this acl portion eventually */
  OUT_AFSUint ("acltype");
  OUT_AFSFid ();
  OUT_AFSHyper ("minVVp");
  OUT_AFSUint ("flags");
  OUT_SEQNUM ();

  return offset;
}
static int
afs4int_dissect_storestatus_rqst (tvbuff_t * tvb, int offset,
				  packet_info * pinfo, proto_tree * tree,
				  char *drep)
{

/*
        [in]    afsFid          *Fidp,
        [in]    afsStoreStatus  *InStatusp,
        [in]    afsHyper        *minVVp,
        [in]    unsigned32      Flags,
*/

  OUT_AFSFid ();
  OUT_AFSStoreStatus ();
  OUT_AFSHyper ("minVVp");
  OUT_AFSUint ("flags");
  OUT_SEQNUM ();

  return offset;
}
static int
afs4int_dissect_createfile_rqst (tvbuff_t * tvb, int offset,
				 packet_info * pinfo, proto_tree * tree,
				 char *drep)
{

/*
        [in]    afsFid          *DirFidp,
        [in]    afsTaggedName   *Namep,
        [in]    afsStoreStatus  *InStatusp,
        [in]    afsHyper        *minVVp,
        [in]    unsigned32      Flags,
*/

  OUT_AFSFid ();
  OUT_AFSTaggedName ();
  OUT_AFSStoreStatus ();
  OUT_AFSHyper ("minVVp");
  OUT_AFSUint ("flags");
  OUT_SEQNUM ();

  return offset;
}
static int
afs4int_dissect_rename_rqst (tvbuff_t * tvb, int offset,
			     packet_info * pinfo, proto_tree * tree,
			     char *drep)
{
/*
        [in]    afsFid          *OldDirFidp,
        [in]    afsFidTaggedName        *OldNamep,
        [in]    afsFid          *NewDirFidp,
        [in]    afsFidTaggedName        *NewNamep,
        [in]    afsHyper        *returnTokenIDp,
        [in]    afsHyper        *minVVp,
        [in]    unsigned32      Flags,
*/

  OUT_AFSFid ();
  OUT_AFSFidTaggedName ();
  OUT_AFSFid ();
  OUT_AFSFidTaggedName ();
  OUT_AFSHyper ("returnTokenIDp");
  OUT_AFSHyper ("minVVp");
  OUT_AFSUint ("flags");
  OUT_SEQNUM ();

  return offset;
}

static int
afs4int_dissect_symlink_rqst (tvbuff_t * tvb, int offset,
			      packet_info * pinfo, proto_tree * tree,
			      char *drep)
{
/*
        [in]    afsFid          *DirFidp,
        [in]    afsTaggedName   *Namep,
        [in]    afsTaggedPath   *LinkContentsp,
        [in]    afsStoreStatus  *InStatusp,
        [in]    afsHyper        *minVVp,
        [in]    unsigned32      Flags,
*/


  OUT_AFSFid ();
  OUT_AFSTaggedName ();
/* XXX need afsTaggedPath added for LinkContentsp*/
  OUT_AFSStoreStatus ();
  OUT_AFSHyper ("minVVp");
  OUT_AFSUint ("flags");

  return offset;
}
static int
afs4int_dissect_readdir_rqst (tvbuff_t * tvb, int offset,
			      packet_info * pinfo, proto_tree * tree,
			      char *drep)
{
/*
        [in]    afsFid          *DirFidp,
        [in]    afsHyper        *Offsetp,
        [in]    unsigned32      Size,
        [in]    afsHyper        *minVVp,
        [in]    unsigned32      Flags,
*/

  OUT_AFSFid ();
  OUT_AFSHyper ("Offsetp");
  OUT_AFSUint ("Size");
  OUT_AFSHyper ("minVVp");
  OUT_AFSUint ("flags");
  OUT_SEQNUM ();

  return offset;
}

static int
afs4int_dissect_makedir_rqst (tvbuff_t * tvb, int offset,
			      packet_info * pinfo, proto_tree * tree,
			      char *drep)
{
/*
        [in]    afsFid          *DirFidp,
        [in]    afsTaggedName   *Namep,
        [in]    afsStoreStatus  *InStatusp,
        [in]    afsHyper        *minVVp,
        [in]    unsigned32      Flags,
*/

  OUT_AFSFid ();
  OUT_AFSTaggedName ();
  OUT_AFSStoreStatus ();
  OUT_AFSHyper ("minVVp");
  OUT_AFSUint ("flags");
  OUT_SEQNUM ();

  return offset;
}

static int
afs4int_dissect_removedir_rqst (tvbuff_t * tvb, int offset,
				packet_info * pinfo, proto_tree * tree,
				char *drep)
{
/*
        [in]    afsFid          *DirFidp,
        [in]    afsFidTaggedName        *Namep,
        [in]    afsHyper        *returnTokenIDp,
        [in]    afsHyper        *minVVp,
        [in]    unsigned32      Flags,
*/

  OUT_AFSFid ();
  OUT_AFSFidTaggedName ();
  OUT_AFSHyper ("returnTokenIDp");
  OUT_AFSHyper ("minVVp");
  OUT_AFSUint ("flags");
  OUT_SEQNUM ();

  return offset;
}

static int
afs4int_dissect_lookup_rqst (tvbuff_t * tvb, int offset,
			     packet_info * pinfo, proto_tree * tree,
			     char *drep)
{
/*
        [in]    afsFid          *DirFidp,
        [in]    afsTaggedName   *Namep,
        [in]    afsHyper        *minVVp,
        [in]    unsigned32      Flags,
*/
  OUT_AFSFid ();
  OUT_AFSTaggedName ();
  OUT_AFSHyper ("minVVp");
  OUT_AFSUint ("flags");
  OUT_SEQNUM ();

  return offset;
}
static int
afs4int_dissect_lookup_resp (tvbuff_t * tvb, int offset,
			     packet_info * pinfo, proto_tree * tree,
			     char *drep)
{
/*
        [out]   afsFid          *OutFidp,
        [out]   afsFetchStatus  *OutFidStatusp,
        [out]   afsFetchStatus  *OutDirStatusp,
        [out]   afsToken        *OutTokenp,
        [out]   afsVolSync      *Syncp
*/

  OUT_AFSFid ();
  OUT_AFSFetchStatus ();
  OUT_AFSFetchStatus ();
  OUT_AFSToken ();
  OUT_AFSVolSync ();
  OUT_ST ();
  OUT_SEQNUM ();

  return offset;

}

static int
afs4int_dissect_makemountpoint_rqst (tvbuff_t * tvb, int offset,
				     packet_info * pinfo, proto_tree * tree,
				     char *drep)
{

/*
        [in]    afsFid          *DirFidp,
        [in]    afsTaggedName   *Namep,
        [in]    afsTaggedName   *cellNamep,
        [in]    afsFStype       Type,
        [in]    afsTaggedName   *volumeNamep,
        [in]    afsStoreStatus  *InStatusp,
        [in]    afsHyper        *minVVp,
        [in]    unsigned32      Flags,
*/

  OUT_AFSFid ();
  OUT_AFSTaggedName ();
  OUT_AFSTaggedName ();
  offset =
    dissect_ndr_uint16 (tvb, offset, pinfo, tree, drep, hf_afs4int_fstype,
			NULL);
  OUT_AFSTaggedName ();
  OUT_AFSStoreStatus ();
  OUT_AFSHyper ("minVVp");
  OUT_AFSUint ("flags");
  OUT_SEQNUM ();

  return offset;
}
static int
afs4int_dissect_setcontext_rqst (tvbuff_t * tvb, int offset,
				 packet_info * pinfo, proto_tree * tree,
				 char *drep)
{

/*
        [in]    handle_t        h,
        [in]    unsigned32      epochTime,
        [in]    afsNetData      *callbackAddr,
        [in]    unsigned32      Flags,
        [in]    afsUUID         *secObjectID,
        [in]    unsigned32      clientSizesAttrs,
        [in]    unsigned32      parm7
*/

  e_uuid_t secobjectid;
  guint32 epochtime, flags, clientsizesattrs, parm7, key_size;
  guint16 type;
  guint8 data0, data1, data2, data3, data4, data5, data6, data7, data8, data9,
    data10, data11, data12, data13, data14;
  char *key_t = NULL;

  offset =
    dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep,
			hf_afs4int_setcontext_rqst_epochtime, &epochtime);
  if (epochtime != 16)
    {
      /*
       * we have found that sctx calls can be entirely devoid of the data field.... 
       */
      offset =
	dissect_ndr_uint16
	(tvb,
	 offset,
	 pinfo,
	 tree, drep, hf_afs4int_setcontext_rqst_afsnetaddr_type, &type);
      offset =
	dissect_ndr_uint8
	(tvb,
	 offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_data0, &data0);
      offset =
	dissect_ndr_uint8
	(tvb,
	 offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_data1, &data1);
      offset =
	dissect_ndr_uint8
	(tvb,
	 offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_data2, &data2);
      offset =
	dissect_ndr_uint8
	(tvb,
	 offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_data3, &data3);
      offset =
	dissect_ndr_uint8
	(tvb,
	 offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_data4, &data4);
      offset =
	dissect_ndr_uint8
	(tvb,
	 offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_data5, &data5);
      offset =
	dissect_ndr_uint8
	(tvb,
	 offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_data6, &data6);
      offset =
	dissect_ndr_uint8
	(tvb,
	 offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_data7, &data7);
      offset =
	dissect_ndr_uint8
	(tvb,
	 offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_data8, &data8);
      offset =
	dissect_ndr_uint8
	(tvb,
	 offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_data9, &data9);
      offset =
	dissect_ndr_uint8
	(tvb,
	 offset,
	 pinfo, tree, drep, hf_afs4int_setcontext_rqst_data10, &data10);
      offset =
	dissect_ndr_uint8
	(tvb,
	 offset,
	 pinfo, tree, drep, hf_afs4int_setcontext_rqst_data11, &data11);
      offset =
	dissect_ndr_uint8
	(tvb,
	 offset,
	 pinfo, tree, drep, hf_afs4int_setcontext_rqst_data12, &data12);
      offset =
	dissect_ndr_uint8
	(tvb,
	 offset,
	 pinfo, tree, drep, hf_afs4int_setcontext_rqst_data13, &data13);
      offset =
	dissect_ndr_uint8
	(tvb,
	 offset,
	 pinfo, tree, drep, hf_afs4int_setcontext_rqst_data14, &data14);
      offset =
	dissect_ndr_uint32
	(tvb,
	 offset,
	 pinfo, tree, drep, hf_afs4int_setcontext_rqst_key_size, &key_size);
      proto_tree_add_string
	(tree,
	 hf_afs4int_setcontext_rqst_key_t,
	 tvb,
	 offset,
	 hf_afs4int_setcontext_rqst_key_size,
	 tvb_get_ptr (tvb, offset, key_size));
      key_t = tvb_get_ptr (tvb, offset, key_size);
      offset += key_size;
      offset =
	dissect_ndr_uint32
	(tvb, offset, pinfo, tree, drep, hf_afs4int_flags, &flags);
      offset =
	dissect_ndr_uuid_t
	(tvb,
	 offset,
	 pinfo,
	 tree, drep, hf_afs4int_setcontext_rqst_secobjectid, &secobjectid);
      offset =
	dissect_ndr_uint32
	(tvb,
	 offset,
	 pinfo,
	 tree, drep,
	 hf_afs4int_setcontext_rqst_clientsizesattrs, &clientsizesattrs);
      offset =
	dissect_ndr_uint32
	(tvb,
	 offset, pinfo, tree, drep, hf_afs4int_setcontext_rqst_parm7, &parm7);
      if (check_col (pinfo->cinfo, COL_INFO))
	col_append_fstr
	  (pinfo->
	   cinfo,
	   COL_INFO,
	   " :Data:%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x",
	   data0,
	   data1,
	   data2,
	   data3,
	   data4,
	   data5,
	   data6,
	   data7, data8, data9, data10, data11, data12, data13, data14);
    }
  else
    {
/* we have some string with a 44 offset.... XXX FIXME */
      offset += 44;
      offset =
	dissect_ndr_uint32
	(tvb,
	 offset,
	 pinfo, tree, drep, hf_afs4int_setcontext_rqst_key_size, &key_size);
      proto_tree_add_string
	(tree,
	 hf_afs4int_setcontext_rqst_key_t,
	 tvb,
	 offset,
	 hf_afs4int_setcontext_rqst_key_size,
	 tvb_get_ptr (tvb, offset, key_size));
      key_t = tvb_get_ptr (tvb, offset, key_size);
      offset += key_size;
      if (check_col (pinfo->cinfo, COL_INFO))
	col_append_fstr (pinfo->cinfo, COL_INFO, " :request for: %s", key_t);
    }

  if (check_col (pinfo->cinfo, COL_INFO))
    col_append_str (pinfo->cinfo, COL_INFO, " Flags=");
  if ((flags & AFS_FLAG_SEC_SERVICE) == AFS_FLAG_SEC_SERVICE)
    {
      if (check_col (pinfo->cinfo, COL_INFO))
	col_append_str (pinfo->cinfo, COL_INFO, ":SEC_SERVICE");
    }
  if ((flags & AFS_FLAG_CONTEXT_NEW_IF) == AFS_FLAG_CONTEXT_NEW_IF)
    {
      if (check_col (pinfo->cinfo, COL_INFO))
	col_append_str (pinfo->cinfo, COL_INFO, ":NEW_IF");
    }
  if ((flags & AFS_FLAG_CONTEXT_DO_RESET) == AFS_FLAG_CONTEXT_DO_RESET)
    {
      if (check_col (pinfo->cinfo, COL_INFO))
	col_append_str (pinfo->cinfo, COL_INFO, ":DO_RESET");
    }
  if ((flags & AFS_FLAG_CONTEXT_NEW_ACL_IF) == AFS_FLAG_CONTEXT_NEW_ACL_IF)
    {
      if (check_col (pinfo->cinfo, COL_INFO))
	col_append_str (pinfo->cinfo, COL_INFO, ":NEW_ACL_IF");
    }
  if ((flags &
       AFS_FLAG_CONTEXT_NEW_TKN_TYPES) == AFS_FLAG_CONTEXT_NEW_TKN_TYPES)
    {
      if (check_col (pinfo->cinfo, COL_INFO))
	col_append_str (pinfo->cinfo, COL_INFO, ":NEW_TKN_TYPES");
    }
  OUT_SEQNUM ();
  return offset;
}

static int
  afs4int_dissect_setcontext_resp
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
  OUT_ST ();
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_lookuproot_resp
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
  /*
   *        [out]   afsFid          *OutFidp,
   *        [out]   afsFetchStatus  *OutFidStatusp,
   *        [out]   afsToken        *OutTokenp,
   *        [out]   afsVolSync      *Syncp
   */
  OUT_AFSFid ();
  OUT_AFSFetchStatus ();
  OUT_AFSToken ();
  OUT_AFSVolSync ();
  OUT_ST ();
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_fetchdata_resp
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{

/*
        [out]   afsFetchStatus  *OutStatusp,
        [out]   afsToken        *OutTokenp,
        [out]   afsVolSync      *Syncp,
        [out]   pipe_t          *fetchStream
*/
  OUT_AFSFetchStatus ();
  OUT_AFSToken ();
  OUT_AFSVolSync ();
/* XXX need pipe_t figured out and inserted here */
  OUT_ST ();
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_fetchacl_resp
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
        [out]   afsACL          *AccessListp,
        [out]   afsFetchStatus  *OutStatusp,
        [out]   afsVolSync      *Syncp
*/

  OUT_AFSUint ("acl_len");
  offset = +acl_len;		/* XXX need to add afsACL here to replace above */
  OUT_AFSFetchStatus ();
  OUT_AFSVolSync ();
  OUT_ST ();
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_fetchstatus_resp
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
        [out]   afsFetchStatus  *OutStatusp,
        [out]   afsToken        *OutTokenp,
        [out]   afsVolSync      *Syncp
*/

  OUT_AFSFetchStatus ();
  OUT_AFSToken ();
  OUT_AFSVolSync ();
  OUT_ST ();
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_storedata_resp
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
        [out]   afsFetchStatus  *OutStatusp,
        [out]   afsVolSync      *Syncp
*/

  OUT_AFSFetchStatus ();
  OUT_AFSVolSync ();
  OUT_ST ();
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_storeacl_resp
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
        [out]   afsFetchStatus  *OutStatusp,
        [out]   afsVolSync      *Syncp
*/

  OUT_AFSFetchStatus ();
  OUT_AFSVolSync ();
  OUT_ST ();
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_storestatus_resp
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
        [out]   afsFetchStatus  *OutStatusp,
        [out]   afsVolSync      *Syncp
*/

  OUT_AFSFetchStatus ();
  OUT_AFSVolSync ();
  OUT_ST ();
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_removefile_resp
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
        [out]   afsFetchStatus  *OutDirStatusp,
        [out]   afsFetchStatus  *OutFileStatusp,
        [out]   afsFid          *OutFileFidp,
        [out]   afsVolSync      *Syncp
*/
  OUT_AFSFetchStatus ();
  OUT_AFSFetchStatus ();
  OUT_AFSFid ();
  OUT_AFSVolSync ();
  OUT_ST ();
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_createfile_resp
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{

/*
        [out]   afsFid          *OutFidp,
        [out]   afsFetchStatus  *OutFidStatusp,
        [out]   afsFetchStatus  *OutDirStatusp,
        [out]   afsToken        *OutTokenp,
        [out]   afsVolSync      *Syncp
*/

  OUT_AFSFid ();
  OUT_AFSFetchStatus ();
  OUT_AFSFetchStatus ();
  OUT_AFSToken ();
  OUT_AFSVolSync ();
  OUT_ST ();
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_rename_resp
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/* 
        [out]   afsFetchStatus  *OutOldDirStatusp,
        [out]   afsFetchStatus  *OutNewDirStatusp,
        [out]   afsFid          *OutOldFileFidp,
        [out]   afsFetchStatus  *OutOldFileStatusp,
        [out]   afsFid          *OutNewFileFidp,
        [out]   afsFetchStatus  *OutNewFileStatusp,
        [out]   afsVolSync      *Syncp
*/

  OUT_AFSFetchStatus ();
  OUT_AFSFetchStatus ();
  OUT_AFSFid ();
  OUT_AFSFetchStatus ();
  OUT_AFSFid ();
  OUT_AFSFetchStatus ();
  OUT_AFSVolSync ();
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_symlink_resp
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
        [out]   afsFid          *OutFidp,
        [out]   afsFetchStatus  *OutFidStatusp,
        [out]   afsFetchStatus  *OutDirStatusp,
        [out]   afsToken        *OutTokenp,
        [out]   afsVolSync      *Syncp
*/
  OUT_AFSFid ();
  OUT_AFSFetchStatus ();
  OUT_AFSFetchStatus ();
  OUT_AFSToken ();
  OUT_AFSVolSync ();
  OUT_ST ();
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_hardlink_resp
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
        [out]   afsFetchStatus  *OutFidStatusp,
        [out]   afsFetchStatus  *OutDirStatusp,
        [out]   afsVolSync      *Syncp
*/


  OUT_AFSFetchStatus ();
  OUT_AFSFetchStatus ();
  OUT_AFSVolSync ();
  OUT_ST ();
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_hardlink_rqst
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
        [in]    afsFid          *DirFidp,
        [in]    afsTaggedName   *Namep,
        [in]    afsFid          *ExistingFidp,
        [in]    afsHyper        *minVVp,
        [in]    unsigned32      Flags,
*/

  OUT_AFSFid ();
  OUT_AFSTaggedName ();
  OUT_AFSFid ();
  OUT_AFSHyper ("minVVp");
  OUT_AFSUint ("flags");
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_makedir_resp
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
        [out]   afsFid          *OutFidp,
        [out]   afsFetchStatus  *OutFidStatusp,
        [out]   afsFetchStatus  *OutDirStatusp,
        [out]   afsToken        *OutTokenp,
        [out]   afsVolSync      *Syncp
*/
  OUT_AFSFid ();
  OUT_AFSFetchStatus ();
  OUT_AFSFetchStatus ();
  OUT_AFSToken ();
  OUT_AFSVolSync ();
  OUT_ST ();
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_removedir_resp
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
        [out]   afsFetchStatus  *OutDirStatusp,
        [out]   afsFid          *OutFidp,
        [out]   afsFetchStatus  *OutDelStatusp,
        [out]   afsVolSync      *Syncp
*/

  OUT_AFSFetchStatus ();
  OUT_AFSFid ();
  OUT_AFSFetchStatus ();
  OUT_AFSVolSync ();
  OUT_ST ();
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_readdir_resp
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
        [out]   afsHyper        *NextOffsetp,
        [out]   afsFetchStatus  *OutDirStatusp,
        [out]   afsToken        *OutTokenp,
        [out]   afsVolSync      *Syncp,
        [out]   pipe_t          *dirStream
*/

  OUT_AFSHyper ("NextOffsetp");
  OUT_AFSFetchStatus ();
  OUT_AFSToken ();
  OUT_AFSVolSync ();
/* XXX need to add pipe_t here, once figured out. */
  OUT_ST ();
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_releasetokens_resp
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
	no out
*/

  OUT_ST ();
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_releasetokens_rqst
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
        [in]    afsReturns      *Tokens_Arrayp,
        [in]    unsigned32      Flags
*/
/* XXX fix with afsReturns */
  OUT_AFSUint ("flags");
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_gettime_resp
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
        [out]   unsigned32      *Secondsp,
        [out]   unsigned32      *USecondsp,
        [out]   unsigned32      *SyncDistance,
        [out]   unsigned32      *SyncDispersion
*/

  OUT_AFSUint ("Secondsp");
  OUT_AFSUint ("USecondsp");
  OUT_AFSUint ("SyncDistance");
  OUT_AFSUint ("SyncDispersion");
  OUT_ST ();
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_gettime_rqst
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/* nothing */

  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_processquota_resp
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
        [in,out]        afsQuota        *quotaListp,
        [out]           afsFetchStatus  *OutStatusp,
        [out]           afsVolSync      *Syncp
*/
/* XXX need afsQuota */
  OUT_AFSFetchStatus ();
  OUT_AFSVolSync ();
  OUT_ST ();
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_processquota_rqst
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
        [in]            afsFid          *Fidp,
        [in]            afsHyper        *minVVp,
        [in]            unsigned32      Flags,
        [in,out]        afsQuota        *quotaListp,
*/

  OUT_AFSFid ();
  OUT_AFSHyper ("minVVp");
  OUT_AFSUint ("flags");
/* XXX need to figure out afsQuota here */
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_getserverinterfaces_rqst
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
        [in, out]               dfs_interfaceList *serverInterfacesP
*/
/* XXX figure out dfs_interfacelist */
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_getserverinterfaces_resp
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
        [in, out]               dfs_interfaceList *serverInterfacesP
*/
/* XXX figure out dfs_interfacelist */

  OUT_ST ();
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_setparams_rqst
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
        [in]            unsigned32      Flags,
        [in, out]       afsConnParams   *paramsP
*/
  OUT_AFSUint ("flags");
/* XXX figure out afsConnParams */
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_setparams_resp
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
        [in, out]       afsConnParams   *paramsP
*/
/* XXX figure out afsConnParams */

  OUT_ST ();
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_makemountpoint_resp
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
        [out]   afsFid          *OutFidp,
        [out]   afsFetchStatus  *OutFidStatusp,
        [out]   afsFetchStatus  *OutDirStatusp,
        [out]   afsVolSync      *Syncp
*/
  OUT_AFSFid ();
  OUT_AFSFetchStatus ();
  OUT_AFSFetchStatus ();
  OUT_AFSVolSync ();
  OUT_ST ();
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_getstatistics_rqst
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
nothing for request
*/
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_getstatistics_resp
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
        [out]   afsStatistics   *Statisticsp
*/
/* XXX figure out afsStatistics */
  OUT_ST ();
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_bulkfetchvv_rqst
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
        [in]    afsHyper        *cellIdp,
        [in]    afsBulkVolIDs   *VolIDsp,
        [in]    unsigned32      NumVols,
        [in]    unsigned32      Flags,
        [in]    unsigned32      spare1,
        [in]    unsigned32      spare2,
*/
  OUT_AFSHyper ("cellIdp");
/* XXX figure out the afsBulkVolIDS */
  OUT_AFSUint ("NumVols");
  OUT_AFSUint ("Flags");
  OUT_AFSUint ("spare1");
  OUT_AFSUint ("spare2");
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_bulkfetchvv_resp
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{

/*
        [out]   afsBulkVVs      *VolVVsp,
        [out]   unsigned32      *spare4
*/
/* XXX need to figure out afsBulkVVs */
  OUT_AFSUint ("spare4");
  OUT_ST ();
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_bulkkeepalive_resp
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
        [out]   unsigned32      *spare4
*/

  OUT_AFSUint ("spare4");
  OUT_ST ();
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_bulkkeepalive_rqst
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
        [in]    afsBulkFEX      *KAFEXp,
        [in]    unsigned32      numExecFids,
        [in]    unsigned32      Flags,
        [in]    unsigned32      spare1,
        [in]    unsigned32      spare2,
*/
/* XXX figure out afsBulkFEX */
  OUT_AFSUint ("numExecFids");
  OUT_AFSUint ("Flags");
  OUT_AFSUint ("spare1");
  OUT_AFSUint ("spare2");
  OUT_SEQNUM ();
  return offset;
}
static int
  afs4int_dissect_bulkfetchstatus_rqst
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{
/*
        [in]            afsFid          *DirFidp,
        [in]            afsHyper        *Offsetp,
        [in]            unsigned32      Size,
        [in]            afsHyper        *minVVp,
        [in]            unsigned32      Flags,
*/
  OUT_AFSFid ();
  OUT_AFSHyper ("Offsetp");
  OUT_AFSUint ("Size");
  OUT_AFSHyper ("minVVp");
  OUT_AFSUint ("Flags");
  OUT_SEQNUM ();
  return offset;
}

static int
  afs4int_dissect_bulkfetchstatus_resp
  (tvbuff_t *
   tvb, int offset, packet_info * pinfo, proto_tree * tree, char *drep)
{

/*
        [out]           BulkStat        *bulkstats,
        [out]           afsHyper        *NextOffsetp,
        [out]           afsFetchStatus  *OutDirStatusp,
        [out]           afsToken        *OutTokenp,
        [out]           afsVolSync      *Syncp,
        [out]           pipe_t          *dirStream
*/

/* XXX figure out BulkStat */
  OUT_AFSHyper ("NextOffSetp");
  OUT_AFSFetchStatus ();
  OUT_AFSToken ();
  OUT_AFSVolSync ();
/* XXX figure out pipe_t */
  OUT_ST ();
  OUT_SEQNUM ();
  return offset;
}

static dcerpc_sub_dissector AFS4Int_dissectors[] = {
  {
   0,
   "DFS_SetContext",
   afs4int_dissect_setcontext_rqst,
   afs4int_dissect_setcontext_resp}
  ,
  {
   1,
   "DFS_LookupRoot",
   afs4int_dissect_lookuproot_rqst,
   afs4int_dissect_lookuproot_resp}
  ,
  {
   2,
   "DFS_FetchData",
   afs4int_dissect_fetchdata_rqst,
   afs4int_dissect_fetchdata_resp}
  ,
  {
   3,
   "DFS_FetchACL",
   afs4int_dissect_fetchacl_rqst,
   afs4int_dissect_fetchacl_resp}
  ,
  {
   4,
   "DFS_FetchStatus",
   afs4int_dissect_fetchstatus_rqst,
   afs4int_dissect_fetchstatus_resp}
  ,
  {
   5,
   "DFS_StoreData",
   afs4int_dissect_storedata_rqst,
   afs4int_dissect_storedata_resp}
  ,
  {
   6,
   "DFS_StoreACL",
   afs4int_dissect_storeacl_rqst,
   afs4int_dissect_storeacl_resp}
  ,
  {
   7,
   "DFS_StoreStatus",
   afs4int_dissect_storestatus_rqst,
   afs4int_dissect_storestatus_resp}
  ,
  {
   8,
   "DFS_RemoveFile",
   afs4int_dissect_removefile_rqst,
   afs4int_dissect_removefile_resp}
  ,
  {
   9,
   "DFS_CreateFile",
   afs4int_dissect_createfile_rqst,
   afs4int_dissect_createfile_resp}
  ,
  {
   10,
   "DFS_Rename",
   afs4int_dissect_rename_rqst,
   afs4int_dissect_rename_resp}
  ,
  {
   11,
   "DFS_Symlink",
   afs4int_dissect_symlink_rqst,
   afs4int_dissect_symlink_resp}
  ,
  {
   12,
   "DFS_HardLink",
   afs4int_dissect_hardlink_rqst,
   afs4int_dissect_hardlink_resp}
  ,
  {
   13,
   "DFS_MakeDir",
   afs4int_dissect_makedir_rqst,
   afs4int_dissect_makedir_resp}
  ,
  {
   14,
   "DFS_RemoveDir",
   afs4int_dissect_removedir_rqst,
   afs4int_dissect_removedir_resp}
  ,
  {
   15,
   "DFS_Readdir",
   afs4int_dissect_readdir_rqst,
   afs4int_dissect_readdir_resp}
  ,
  {
   16,
   "DFS_Lookup",
   afs4int_dissect_lookup_rqst,
   afs4int_dissect_lookup_resp}
  ,
  {
   17,
   "DFS_GetToken",
   afs4int_dissect_gettoken_rqst,
   afs4int_dissect_gettoken_resp}
  ,
  {
   18,
   "DFS_ReleaseTokens",
   afs4int_dissect_releasetokens_rqst,
   afs4int_dissect_releasetokens_resp}
  ,
  {
   19,
   "DFS_GetTime",
   afs4int_dissect_gettime_rqst,
   afs4int_dissect_gettime_resp}
  ,
  {
   20,
   "DFS_MakeMountPoint",
   afs4int_dissect_makemountpoint_rqst,
   afs4int_dissect_makemountpoint_resp}
  ,
  {
   21,
   "DFS_GetStatistics",
   afs4int_dissect_getstatistics_rqst,
   afs4int_dissect_getstatistics_resp}
  ,
  {
   22,
   "DFS_BulkFetchVV",
   afs4int_dissect_bulkfetchvv_rqst,
   afs4int_dissect_bulkfetchvv_resp}
  ,
  {
   23,
   "DFS_BulkKeepAlive",
   afs4int_dissect_bulkkeepalive_rqst,
   afs4int_dissect_bulkkeepalive_resp}
  ,
  {
   24,
   "DFS_ProcessQuota",
   afs4int_dissect_processquota_rqst,
   afs4int_dissect_processquota_resp}
  ,
  {
   25,
   "DFS_GetServerInterfaces",
   afs4int_dissect_getserverinterfaces_rqst,
   afs4int_dissect_getserverinterfaces_resp}
  ,
  {
   26,
   "DFS_SetParams",
   afs4int_dissect_setparams_rqst,
   afs4int_dissect_setparams_resp}
  ,
  {
   27,
   "DFS_BulkFetchStatus",
   afs4int_dissect_bulkfetchstatus_rqst,
   afs4int_dissect_bulkfetchstatus_resp}
  ,
  {
   0, NULL,
   NULL, NULL}
  ,
};
void
proto_register_AFS4Int (void)
{

  static hf_register_info hf[] = {
    {
     &hf_afs4int_setcontext_rqst_epochtime,
     {
      "EpochTime:",
      "hf_afs4int_setcontext_rqst_epochtime",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_setcontext_rqst_afsnetaddr_type,
     {
      "AfsNetAddr_Type:",
      "hf_afs4int_setcontext_rqst_afsnetaddr_type",
      FT_UINT16,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_setcontext_rqst_key_t,
     {
      "Key_t:",
      "hf_afs4int_setcontext_rqst_key_t",
      FT_STRING,
      BASE_NONE,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_setcontext_rqst_key_size,
     {
      "Key_Size:",
      "hf_afs4int_setcontext_rqst_key_size",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_setcontext_rqst_secobjectid,
     {
      "SetObjectid:",
      "hf_afs4int_setcontext_rqst_secobjectid",
      FT_STRING,
      BASE_NONE,
      NULL, 0x0,
      "UUID",
      HFILL}
     }
    ,
    {
     &hf_afs4int_setcontext_rqst_clientsizesattrs,
     {
      "ClientSizeAttrs:",
      "hf_afs4int_setcontext_rqst_clientsizesattrs",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_setcontext_rqst_parm7,
     {
      "Parm7:",
      "hf_afs4int_setcontext_rqst_parm7",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_setcontext_rqst_data0,
     {
      "Data 0:",
      "hf_afs4int_setcontext_rqst_data0",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_setcontext_rqst_data1,
     {
      "Data 1:",
      "hf_afs4int_setcontext_rqst_data1",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_setcontext_rqst_data2,
     {
      "Data 2:",
      "hf_afs4int_setcontext_rqst_data2",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_setcontext_rqst_data3,
     {
      "Data 3:",
      "hf_afs4int_setcontext_rqst_data3",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_setcontext_rqst_data4,
     {
      "Data 4:",
      "hf_afs4int_setcontext_rqst_data4",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_setcontext_rqst_data5,
     {
      "Data 5:",
      "hf_afs4int_setcontext_rqst_data5",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_setcontext_rqst_data6,
     {
      "Data 6:",
      "hf_afs4int_setcontext_rqst_data6",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_setcontext_rqst_data7,
     {
      "Data 7:",
      "hf_afs4int_setcontext_rqst_data7",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_setcontext_rqst_data8,
     {
      "Data 8:",
      "hf_afs4int_setcontext_rqst_data8",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_setcontext_rqst_data9,
     {
      "Data 9:",
      "hf_afs4int_setcontext_rqst_data9",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_setcontext_rqst_data10,
     {
      "Data 10:",
      "hf_afs4int_setcontext_rqst_data10",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_setcontext_rqst_data11,
     {
      "Data 11:",
      "hf_afs4int_setcontext_rqst_data11",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_setcontext_rqst_data12,
     {
      "Data 12:",
      "hf_afs4int_setcontext_rqst_data12",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_setcontext_rqst_data13,
     {
      "Data 13:",
      "hf_afs4int_setcontext_rqst_data13",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_setcontext_rqst_data14,
     {
      "Data 14:",
      "hf_afs4int_setcontext_rqst_data14",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_setcontext_resp_st,
     {
      "hf_afs4int_setcontext_resp_st:",
      "hf_afs4int_setcontext_resp_st",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    { &hf_afs4int_flags, { "hf_afs4int_flags", "hf_afs4int_flags", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL} },
    { &hf_afs4int_acl_len, { "hf_afs4int_acl_len", "hf_afs4int_acl_len", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL} } ,
    { &hf_afs4int_acltype, {  "hf_afs4int_acltype", "hf_afs4int_acltype", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL} },
    { &hf_afs4int_minvvp_high, { "hf_afs4int_minvvp_high", "hf_afs4int_minvvp_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL} } ,
    { &hf_afs4int_minvvp_low, { "hf_afs4int_minvvp_low", "hf_afs4int_minvvp_low", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL} } ,
    { &hf_afs4int_cell_high, { "hf_afs4int_cell_high", "hf_afs4int_cell_high", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL} } ,
    {
     &hf_afs4int_cell_low,
     {
      "hf_afs4int_cell_low",
      "hf_afs4int_cell_low",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_volume_low,
     {
      "hf_afs4int_volume_low",
      "hf_afs4int_volume_low",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_volume_high,
     {
      "hf_afs4int_volume_high",
      "hf_afs4int_volume_high",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_vnode,
     {
      "hf_afs4int_vnode",
      "hf_afs4int_vnode",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_unique,
     {
      "hf_afs4int_unique",
      "hf_afs4int_unique",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_accesstime_msec,
     {
      "hf_afs4int_accesstime_msec",
      "hf_afs4int_accesstime_msec",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_accesstime_sec,
     {
      "hf_afs4int_accesstime_sec",
      "hf_afs4int_accesstime_sec",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_aclexpirationtime,
     {
      "hf_afs4int_aclexpirationtime",
      "hf_afs4int_aclexpirationtime",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_agtypeunique,
     {
      "hf_afs4int_agtypeunique",
      "hf_afs4int_agtypeunique",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_anonymousaccess,
     {
      "hf_afs4int_anonymousaccess",
      "hf_afs4int_anonymousaccess",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_author,
     {
      "hf_afs4int_author",
      "hf_afs4int_author",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_blocksused,
     {
      "hf_afs4int_blocksused",
      "hf_afs4int_blocksused",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_calleraccess,
     {
      "hf_afs4int_calleraccess",
      "hf_afs4int_calleraccess",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_changetime_msec,
     {
      "hf_afs4int_changetime_msec",
      "hf_afs4int_changetime_msec",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_changetime_sec,
     {
      "hf_afs4int_changetime_sec",
      "hf_afs4int_changetime_sec",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_clientspare1,
     {
      "hf_afs4int_clientspare1",
      "hf_afs4int_clientspare1",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_dataversion_high,
     {
      "hf_afs4int_dataversion_high",
      "hf_afs4int_dataversion_high",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_dataversion_low,
     {
      "hf_afs4int_dataversion_low",
      "hf_afs4int_dataversion_low",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_devicenumber,
     {
      "hf_afs4int_devicenumber",
      "hf_afs4int_devicenumber",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_devicenumberhighbits,
     {
      "hf_afs4int_devicenumberhighbits",
      "hf_afs4int_devicenumberhighbits",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_filetype,
     {
      "hf_afs4int_filetype",
      "hf_afs4int_filetype",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_group,
     {
      "hf_afs4int_group",
      "hf_afs4int_group",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_himaxspare,
     {
      "hf_afs4int_himaxspare",
      "hf_afs4int_himaxspare",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_interfaceversion,
     {
      "hf_afs4int_interfaceversion",
      "hf_afs4int_interfaceversion",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_length_high,
     {
      "hf_afs4int_length_high",
      "hf_afs4int_length_high",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_length_low,
     {
      "hf_afs4int_length_low",
      "hf_afs4int_length_low",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_linkcount,
     {
      "hf_afs4int_linkcount",
      "hf_afs4int_linkcount",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_lomaxspare,
     {
      "hf_afs4int_lomaxspare",
      "hf_afs4int_lomaxspare",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_mode,
     {
      "hf_afs4int_mode",
      "hf_afs4int_mode",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_modtime_msec,
     {
      "hf_afs4int_modtime_msec",
      "hf_afs4int_modtime_msec",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_modtime_sec,
     {
      "hf_afs4int_modtime_sec",
      "hf_afs4int_modtime_sec",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_objectuuid,
     {
      "hf_afs4int_objectuuid",
      "hf_afs4int_objectuuid",
      FT_STRING,
      BASE_NONE,
      NULL, 0x0,
      "UUID",
      HFILL}
     }
    ,
    {
     &hf_afs4int_owner,
     {
      "hf_afs4int_owner",
      "hf_afs4int_owner",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_parentunique,
     {
      "hf_afs4int_parentunique",
      "hf_afs4int_parentunique",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_parentvnode,
     {
      "hf_afs4int_parentvnode",
      "hf_afs4int_parentvnode",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_pathconfspare,
     {
      "hf_afs4int_pathconfspare",
      "hf_afs4int_pathconfspare",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_servermodtime_msec,
     {
      "hf_afs4int_servermodtime_msec",
      "hf_afs4int_servermodtime_msec",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_servermodtime_sec,
     {
      "hf_afs4int_servermodtime_sec",
      "hf_afs4int_servermodtime_sec",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_spare4,
     {
      "hf_afs4int_spare4",
      "hf_afs4int_spare4",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_spare5,
     {
      "hf_afs4int_spare5",
      "hf_afs4int_spare5",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_spare6,
     {
      "hf_afs4int_spare6",
      "hf_afs4int_spare6",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_typeuuid,
     {
      "hf_afs4int_typeuuid",
      "hf_afs4int_typeuuid",
      FT_STRING,
      BASE_NONE,
      NULL, 0x0,
      "UUID",
      HFILL}
     }
    ,
    {
     &hf_afs4int_volid_hi,
     {
      "hf_afs4int_volid_hi",
      "hf_afs4int_volid_hi",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_volid_low,
     {
      "hf_afs4int_volid_low",
      "hf_afs4int_volid_low",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_vvage,
     {
      "hf_afs4int_vvage",
      "hf_afs4int_vvage",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_vv_hi,
     {
      "hf_afs4int_vv_hi",
      "hf_afs4int_vv_hi",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_vv_low,
     {
      "hf_afs4int_vv_low",
      "hf_afs4int_vv_low",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_vvpingage,
     {
      "hf_afs4int_vvpingage",
      "hf_afs4int_vvpingage",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_vvspare1,
     {
      "hf_afs4int_vvspare1",
      "hf_afs4int_vvspare1",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_vvspare2,
     {
      "hf_afs4int_vvspare2",
      "hf_afs4int_vvspare2",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_beginrange,
     {
      "hf_afs4int_beginrange",
      "hf_afs4int_beginrange",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_beginrangeext,
     {
      "hf_afs4int_beginrangeext",
      "hf_afs4int_beginrangeext",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_endrange,
     {
      "hf_afs4int_endrange",
      "hf_afs4int_endrange",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_endrangeext,
     {
      "hf_afs4int_endrangeext",
      "hf_afs4int_endrangeext",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_expirationtime,
     {
      "hf_afs4int_expirationtime",
      "hf_afs4int_expirationtime",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_tokenid_hi,
     {
      "hf_afs4int_tokenid_hi",
      "hf_afs4int_tokenid_hi",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_tokenid_low,
     {
      "hf_afs4int_tokenid_low",
      "hf_afs4int_tokenid_low",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_type_hi,
     {
      "hf_afs4int_type_hi",
      "hf_afs4int_type_hi",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_type_low,
     {
      "hf_afs4int_type_low",
      "hf_afs4int_type_low",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_tn_length,
     {
      "hf_afs4int_tn_length",
      "hf_afs4int_tn_length",
      FT_UINT16,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_tn_size,
     {
      "hf_afs4int_tn_size",
      "hf_afs4int_tn_size",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_tn_string,
     {
      "hf_afs4int_tn_string",
      "hf_afs4int_tn_string",
      FT_STRING,
      BASE_NONE,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_tn_tag,
     {
      "hf_afs4int_tn_tag",
      "hf_afs4int_tn_tag",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_storestatus_accesstime_sec,
     {
      "hf_afs4int_storestatus_accesstime_sec",
      "hf_afs4int_storestatus_accesstime_sec",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_storestatus_accesstime_usec,
     {
      "hf_afs4int_storestatus_accesstime_usec",
      "hf_afs4int_storestatus_accesstime_usec",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_storestatus_changetime_sec,
     {
      "hf_afs4int_storestatus_changetime_sec",
      "hf_afs4int_storestatus_changetime_sec",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_storestatus_changetime_usec,
     {
      "hf_afs4int_storestatus_changetime_usec",
      "hf_afs4int_storestatus_changetime_usec",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_storestatus_clientspare1,
     {
      "hf_afs4int_storestatus_clientspare1",
      "hf_afs4int_storestatus_clientspare1",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_storestatus_cmask,
     {
      "hf_afs4int_storestatus_cmask",
      "hf_afs4int_storestatus_cmask",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_storestatus_devicenumber,
     {
      "hf_afs4int_storestatus_devicenumber",
      "hf_afs4int_storestatus_devicenumber",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_storestatus_devicenumberhighbits,
     {
      "hf_afs4int_storestatus_devicenumberhighbits",
      "hf_afs4int_storestatus_devicenumberhighbits",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_storestatus_devicetype,
     {
      "hf_afs4int_storestatus_devicetype",
      "hf_afs4int_storestatus_devicetype",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_storestatus_group,
     {
      "hf_afs4int_storestatus_group",
      "hf_afs4int_storestatus_group",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_storestatus_length_high,
     {
      "hf_afs4int_storestatus_length_high",
      "hf_afs4int_storestatus_length_high",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_storestatus_length_low,
     {
      "hf_afs4int_storestatus_length_low",
      "hf_afs4int_storestatus_length_low",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_storestatus_mask,
     {
      "hf_afs4int_storestatus_mask",
      "hf_afs4int_storestatus_mask",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_storestatus_mode,
     {
      "hf_afs4int_storestatus_mode",
      "hf_afs4int_storestatus_mode",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_storestatus_modtime_sec,
     {
      "hf_afs4int_storestatus_modtime_sec",
      "hf_afs4int_storestatus_modtime_sec",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_storestatus_modtime_usec,
     {
      "hf_afs4int_storestatus_modtime_usec",
      "hf_afs4int_storestatus_modtime_usec",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_storestatus_owner,
     {
      "hf_afs4int_storestatus_owner",
      "hf_afs4int_storestatus_owner",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_storestatus_spare1,
     {
      "hf_afs4int_storestatus_spare1",
      "hf_afs4int_storestatus_spare1",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_storestatus_spare2,
     {
      "hf_afs4int_storestatus_spare2",
      "hf_afs4int_storestatus_spare2",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_storestatus_spare3,
     {
      "hf_afs4int_storestatus_spare3",
      "hf_afs4int_storestatus_spare3",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_storestatus_spare4,
     {
      "hf_afs4int_storestatus_spare4",
      "hf_afs4int_storestatus_spare4",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_storestatus_spare5,
     {
      "hf_afs4int_storestatus_spare5",
      "hf_afs4int_storestatus_spare5",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_storestatus_spare6,
     {
      "hf_afs4int_storestatus_spare6",
      "hf_afs4int_storestatus_spare6",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_storestatus_trunc_high,
     {
      "hf_afs4int_storestatus_trunc_high",
      "hf_afs4int_storestatus_trunc_high",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_storestatus_trunc_low,
     {
      "hf_afs4int_storestatus_trunc_low",
      "hf_afs4int_storestatus_trunc_low",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_storestatus_typeuuid,
     {
      "hf_afs4int_storestatus_typeuuid",
      "hf_afs4int_storestatus_typeuuid",
      FT_STRING,
      BASE_NONE,
      NULL, 0x0,
      "UUID",
      HFILL}
     }
    ,
    {
     &hf_afs4int_st,
     {
      "hf_afs4int_st",
      "hf_afs4int_st",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_hyper_high,
     {
      "hf_afs4int_hyper_high",
      "hf_afs4int_hyper_high",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_hyper_low,
     {
      "hf_afs4int_hyper_low",
      "hf_afs4int_hyper_low",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {&hf_afs4int_uint,
     {"hf_afs4int_uint", "hf_afs4int_uint", FT_UINT32, BASE_DEC, NULL, 0x0,
      "", HFILL}},
    {&hf_afs4int_int,
     {"hf_afs4int_int", "hf_afs4int_int", FT_INT32, BASE_DEC, NULL, 0x0, "",
      HFILL}},
    {
     &hf_afs4int_l_end_pos,
     {
      "hf_afs4int_l_end_pos",
      "hf_afs4int_l_end_pos",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_l_end_pos_ext,
     {
      "hf_afs4int_l_end_pos_ext",
      "hf_afs4int_l_end_pos_ext",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_l_fstype,
     {
      "hf_afs4int_l_fstype",
      "hf_afs4int_l_fstype",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_l_pid,
     {
      "hf_afs4int_l_pid",
      "hf_afs4int_l_pid",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_l_start_pos,
     {
      "hf_afs4int_l_start_pos",
      "hf_afs4int_l_start_pos",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_l_start_pos_ext,
     {
      "hf_afs4int_l_start_pos_ext",
      "hf_afs4int_l_start_pos_ext",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_l_sysid,
     {
      "hf_afs4int_l_sysid",
      "hf_afs4int_l_sysid",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_l_type,
     {
      "hf_afs4int_l_type",
      "hf_afs4int_l_type",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
    {
     &hf_afs4int_l_whence,
     {
      "hf_afs4int_l_whence",
      "hf_afs4int_l_whence",
      FT_UINT32,
      BASE_DEC,
      NULL, 0x0,
      "", HFILL}
     }
    ,
  };
  static gint *ett[] = {
    &ett_AFS4Int,
  };
  proto_AFS4Int = proto_register_protocol ("DFS Calls", "DCE_DFS", "dce_dfs");
  proto_register_field_array (proto_AFS4Int, hf, array_length (hf));
  proto_register_subtree_array (ett, array_length (ett));
}

void
proto_reg_handoff_AFS4Int (void)
{
  /*
   * Register the protocol as dcerpc 
   */
  dcerpc_init_uuid
    (proto_AFS4Int,
     ett_AFS4Int, &uuid_AFS4Int, ver_AFS4Int, AFS4Int_dissectors, -1);
}
