/* packet-dcerpc-macros.h
 * Helper macros for DFS packet dissection
 * Copyright 2002, Jaime Fournier <jafour1@yahoo.com>
 *
 * $Id: packet-dcerpc-macros.h,v 1.6 2002/10/27 07:21:55 root Exp root $
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@ethereal.com>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/* Vars for below functions */
guint32 cell_high, cell_low, volume_high, volume_low, vnode, unique, inode;
guint64 volume, cell, afs4int_hyper, type, tokenid;
e_uuid_t typeuuid, objectuuid;
guint32 tn_tag;
guint16 tn_length;
char *tn_string;
guint32 interfaceversion, filetype, linkcount, length_high, length_low,
     dataversion_high, dataversion_low, author, owner, group, calleraccess,
     anonymousaccess, aclexpirationtime, mode, parentvnode, parentunique,
     modtime_sec, modtime_msec, changetime_sec, changetime_msec,
     accesstime_sec, accesstime_msec, servermodtime_sec, servermodtime_msec,
     devicenumber, blocksused, clientspare1, devicenumberhighbits,
     agtypeunique, himaxspare, lomaxspare, pathconfspare, spare1, spare2, spare3, spare4, spare5,
     spare6, flags, minvvp_high, minvvp_low, st, hyper_high, hyper_low, afs4int_uint, afs4int_int, acl_len;


guint32 mask, modtime_sec, modtime_usec, accesstime_sec, accesstime_usec, changetime_sec, changetime_usec,
     owner, group, mode, trunc_high, trunc_low, length_high,
     length_low, devicetype, devicenumber, cmask, cleintspare1, devicenumberhighbits;

guint32 volid_hi, volid_low, vv_hi, vv_low, vvage, vvpingage, vvspare1, vvspare2;
guint tokenid_hi, tokenid_low, expirationtime, type_hi, type_low, beginrange, endrange, beginrangeext, endrangeext;
guint16 l_type, l_whence;
guint32 l_start_pos, l_end_pos, l_pid, l_sysid, l_fstype, l_start_pos_ext, l_end_pos_ext, acltype;


/* Spit out seq_num on COL_INFO macro */

#define OUT_SEQNUM(void) \
  if (check_col (pinfo->cinfo, COL_INFO))  \
    col_append_fstr (pinfo->cinfo, COL_INFO, ": seq_num: %u", master_seqnum);


#define OUT_AFSFid(void) \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_cell_high, &cell_high); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_cell_low, &cell_low); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_volume_high, &volume_high); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep,  hf_afs4int_volume_low, &volume_low); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_vnode, &vnode); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep,  hf_afs4int_unique, &unique);  \
  volume = volume_high | volume_low; \
  cell = cell_high | cell_low; \
  if (check_col (pinfo->cinfo, COL_INFO)) \
    col_append_fstr (pinfo->cinfo, COL_INFO, " :FSID:%llu", volume); \
  if (vnode == 1) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_str (pinfo->cinfo, COL_INFO, " InFS "); \
    } \
  else \
    { \
      inode = ((volume << 16) + vnode) & 0x7fffffff; \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_fstr (pinfo->cinfo, COL_INFO, " inode:%u ", inode); \
    }

e_uuid_t typeuuid, objectuuid;
#define OUT_AFSFetchStatus(void) \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_interfaceversion, &interfaceversion); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_filetype, &filetype); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_linkcount, &linkcount); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_length_high, &length_high); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_length_low, &length_low); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_dataversion_high, &dataversion_high); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_dataversion_low, &dataversion_low); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_author, &author); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_owner, &owner); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_group, &group); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_calleraccess, &calleraccess); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_anonymousaccess, &anonymousaccess); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_aclexpirationtime, &aclexpirationtime); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_mode, &mode); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_parentvnode, &parentvnode); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_parentunique, &parentunique); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_modtime_sec, &modtime_sec); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_modtime_msec, &modtime_msec); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_changetime_sec, &changetime_sec); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_changetime_msec, &changetime_msec); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_accesstime_sec, &accesstime_sec); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_accesstime_msec, &accesstime_msec); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_servermodtime_sec, &servermodtime_sec); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_servermodtime_msec, &servermodtime_msec); \
      offset = dissect_ndr_uuid_t (tvb, offset, pinfo, tree, drep, hf_afs4int_typeuuid, &typeuuid); \
      offset = dissect_ndr_uuid_t (tvb, offset, pinfo, tree, drep, hf_afs4int_objectuuid, &objectuuid); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_devicenumber, &devicenumber); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_blocksused, &blocksused);  \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_clientspare1, &clientspare1); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_devicenumberhighbits, &devicenumberhighbits); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_agtypeunique, &agtypeunique); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_himaxspare, &himaxspare); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_lomaxspare, &lomaxspare); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_pathconfspare, &pathconfspare); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_spare4, &spare4); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_spare5, &spare5); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_spare6, &spare6); \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_fstr (pinfo->cinfo, COL_INFO, " :interfacever:%u filetype:%u linkcount:%u length:%u dataver:%u author:%u owner:%u group:%u calleraccess:%u anonaccess:%u aclexpire:%u mode:%u parentvnode:%u parentunique:%u modtimesec:%u changetime_sec:%u servermodtimesec:%u devicenumber:%u blocksused:%u clientspare:%u devicehighbits:%u agtypeunique:%u", interfaceversion, filetype, linkcount, length_low, dataversion_low, author, owner, group, calleraccess, anonymousaccess, aclexpirationtime, mode, parentvnode, parentunique, modtime_sec, changetime_sec, accesstime_sec, servermodtime_sec, typeuuid, blocksused, clientspare1, devicenumberhighbits, agtypeunique); 

#define OUT_AFSVolSync(void) \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_volid_hi, &volid_hi); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_volid_low, &volid_low); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_vv_hi, &vv_hi); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_vv_low, &vv_low); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_vvage, &vvage); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_vvpingage, &vvpingage); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_vvspare1, &vvspare1); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_vvspare2, &vvspare2);

#define OUT_AFSToken(void) \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_tokenid_hi, &tokenid_hi); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_tokenid_low, &tokenid_low); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_expirationtime, &expirationtime); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_type_hi, &type_hi); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_type_low, &type_low); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_beginrange, &beginrange); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_endrange, &endrange); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_beginrangeext, &beginrangeext); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_endrangeext, &endrangeext); \
  tokenid = tokenid_hi, tokenid_low; \
  type = type_hi | type_low; \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_fstr (pinfo->cinfo, COL_INFO, "  :Tokenid:%llu ExpirationTime:%u beginrange:%u endrange:%u beginrangeext:%u endrangeext:%u", tokenid, expirationtime, beginrange, endrange, beginrangeext, endrangeext ); \
  if (check_col (pinfo->cinfo, COL_INFO)) \
    col_append_str (pinfo->cinfo, COL_INFO, " Type="); \
  if ((type & TKN_LOCK_READ) == TKN_LOCK_READ) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_str (pinfo->cinfo, COL_INFO, ":LOCK_READ"); \
    } \
  if ((type & TKN_LOCK_WRITE) == TKN_LOCK_WRITE) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO))  \
        col_append_str (pinfo->cinfo, COL_INFO, ":LOCK_WRITE"); \
    } \
  if ((type & TKN_DATA_READ) == TKN_DATA_READ) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_str (pinfo->cinfo, COL_INFO, ":DATA_READ"); \
    } \
  if ((type & TKN_DATA_WRITE) == TKN_DATA_WRITE) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_str (pinfo->cinfo, COL_INFO, ":DATA_WRITE"); \
    } \
  if ((type & TKN_OPEN_READ) == TKN_OPEN_READ) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_str (pinfo->cinfo, COL_INFO, ":OPEN_READ"); \
    } \
  if ((type & TKN_OPEN_WRITE) == TKN_OPEN_WRITE) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_str (pinfo->cinfo, COL_INFO, ":OPEN_WRITE");  \
    } \
  if ((type & TKN_OPEN_SHARED) == TKN_OPEN_SHARED) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_str (pinfo->cinfo, COL_INFO, ":OPEN_SHARED"); \
    } \
  if ((type & TKN_OPEN_EXCLUSIVE) == TKN_OPEN_EXCLUSIVE) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_str (pinfo->cinfo, COL_INFO, ":OPEN_EXCLUSIVE"); \
    } \
  if ((type & TKN_OPEN_DELETE) == TKN_OPEN_DELETE) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_str (pinfo->cinfo, COL_INFO, ":OPEN_DELETE"); \
    } \
  if ((type & TKN_OPEN_PRESERVE) == TKN_OPEN_PRESERVE) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_str (pinfo->cinfo, COL_INFO, ":OPEN_PRESERVE"); \
    } \
  if ((type & TKN_STATUS_READ) == TKN_STATUS_READ) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_str (pinfo->cinfo, COL_INFO, ":STATUS_READ"); \
    } \
  if ((type & TKN_STATUS_WRITE) == TKN_STATUS_WRITE) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_str (pinfo->cinfo, COL_INFO, ":STATUS_WRITE"); \
    } \
  if ((type & TKN_OPEN_UNLINK) == TKN_OPEN_UNLINK) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_str (pinfo->cinfo, COL_INFO, ":OPEN_UNLINK"); \
    } \
  if ((type & TKN_SPOT_HERE) == TKN_SPOT_HERE) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_str (pinfo->cinfo, COL_INFO, ":SPOT_HERE"); \
    } \
  if ((type & TKN_SPOT_THERE) == TKN_SPOT_THERE) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_str (pinfo->cinfo, COL_INFO, ":SPOT_THERE"); \
    } \
  if ((type & TKN_OPEN_NO_READ) == TKN_OPEN_NO_READ) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_str (pinfo->cinfo, COL_INFO, ":OPEN_NO_READ"); \
    } \
  if ((type & TKN_OPEN_NO_WRITE) == TKN_OPEN_NO_WRITE) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_str (pinfo->cinfo, COL_INFO, ":OPEN_NO_WRITE"); \
    } \
  if ((type & TKN_OPEN_NO_UNLINK) == TKN_OPEN_NO_UNLINK) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_str (pinfo->cinfo, COL_INFO, ":OPEN_NO_UNLINK"); \
    }

#define OUT_AFSTaggedName(void) \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_tn_tag, &tn_tag);  \
  offset = dissect_ndr_uint16 (tvb, offset, pinfo, tree, drep, hf_afs4int_tn_length, &tn_length);  \
  proto_tree_add_string (tree, hf_afs4int_tn_string, tvb, offset, hf_afs4int_tn_size, tvb_get_ptr (tvb, offset, tn_length)); \
  tn_string = tvb_get_ptr (tvb, offset, tn_length); \
  offset += tn_length; \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_fstr (pinfo->cinfo, COL_INFO, " :tn_tag: %s", tn_string);

#define OUT_AFSFidTaggedName(void) \
	OUT_AFSFid(); \
	OUT_AFSTaggedName();

#define OUT_AFSStoreStatus(void) \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_mask, &mask); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_modtime_sec, &modtime_sec); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_modtime_usec, &modtime_usec); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_accesstime_sec, &accesstime_sec); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_accesstime_usec, &accesstime_usec); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_changetime_sec, &changetime_sec); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_changetime_usec, &changetime_usec); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_owner, &owner); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_group, &group); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_mode, &mode); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_trunc_high, &trunc_high); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_trunc_low, &trunc_low); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_length_high, &length_high); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_length_low, &length_low); \
  offset = dissect_ndr_uuid_t (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_typeuuid, &typeuuid); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_devicetype, &devicetype); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_devicenumber, &devicenumber); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_cmask, &cmask); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_clientspare1, &clientspare1); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_devicenumberhighbits, &devicenumberhighbits); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_spare1, &spare1); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_spare2, &spare2); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_spare3, &spare3); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_spare4, &spare4); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_spare5, &spare5); \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_storestatus_spare6, &spare6);  \
  if (check_col (pinfo->cinfo, COL_INFO)) col_append_fstr (pinfo->cinfo, COL_INFO, " Mask="); \
  if ((mask & AFS_SETMODTIME) == AFS_SETMODTIME) \
 { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_fstr (pinfo->cinfo, COL_INFO, ":SETMODTIME-%u.%u", \
                         modtime_sec, modtime_usec); \
    } \
  if ((mask & AFS_SETOWNER) == AFS_SETOWNER) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_fstr (pinfo->cinfo, COL_INFO, ":SETOWNER-%u", owner); \
    } \
  if ((mask & AFS_SETGROUP) == AFS_SETGROUP) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_fstr (pinfo->cinfo, COL_INFO, ":SETGROUP-%u", group); \
    } \
  if ((mask & AFS_SETMODE) == AFS_SETMODE) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_fstr (pinfo->cinfo, COL_INFO, ":SETMODE-%o", mode); \
    } \
  if ((mask & AFS_SETACCESSTIME) == AFS_SETACCESSTIME) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_fstr (pinfo->cinfo, COL_INFO, ":SETACCESSTIME-%u.%u", \
                         accesstime_sec, accesstime_usec); \
    } \
  if ((mask & AFS_SETCHANGETIME) == AFS_SETCHANGETIME) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_fstr (pinfo->cinfo, COL_INFO, ":SETCHANGETIME-%u.%u", \
                         changetime_sec, changetime_usec); \
    } \
  if ((mask & AFS_SETLENGTH) == AFS_SETLENGTH) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_str (pinfo->cinfo, COL_INFO, ":SETLENGTH"); \
    } \
  if ((mask & AFS_SETTYPEUUID) == AFS_SETTYPEUUID) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_str (pinfo->cinfo, COL_INFO, ":SETTYPEUUID"); \
    } \
  if ((mask & AFS_SETDEVNUM) == AFS_SETDEVNUM) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_str (pinfo->cinfo, COL_INFO, ":SETDEVNUM"); \
    } \
  if ((mask & AFS_SETMODEXACT) == AFS_SETMODEXACT) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_str (pinfo->cinfo, COL_INFO, ":SETMODEXACT"); \
    } \
  if ((mask & AFS_SETTRUNCLENGTH) == AFS_SETTRUNCLENGTH) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_str (pinfo->cinfo, COL_INFO, ":SETTRUNCLENGTH"); \
    } \
  if ((mask & AFS_SETCLIENTSPARE) == AFS_SETCLIENTSPARE) \
    { \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_str (pinfo->cinfo, COL_INFO, ":SETCLIENTSPARE"); \
    }

#define OUT_ST(void) \
  offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_st, &st); \
  /* offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_st, &st); */ \
  if (check_col (pinfo->cinfo, COL_INFO)) \
    col_append_fstr (pinfo->cinfo, COL_INFO, " st:%s ", \
                     val_to_str (st, dce_error_vals, "%u"));

#define OUT_AFSHyper(name) \
 offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_hyper_high, &hyper_high); \
 offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_hyper_low, &hyper_low); \
      afs4int_hyper = hyper_high | hyper_low; \
      if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_fstr (pinfo->cinfo, COL_INFO, " :%s:%llu", name, afs4int_hyper); 



/* proto_tree_add_uint(tree, name, tvb, offset, sizeof(guint32), tvb_get_ntohl(tvb, offset)); \ */

#define OUT_AFSUint(name) \
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_uint, &afs4int_uint); \
        if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_fstr (pinfo->cinfo, COL_INFO, " :%s:%u", name, afs4int_uint); 

/* proto_tree_add_int(tree, name, tvb, offset, sizeof(gint32), tvb_get_ntohl(tvb, offset)); \ */
#define OUT_AFSInt(name) \
        offset = dissect_ndr_int32 (tvb, offset, pinfo, tree, drep, hf_afs4int_int, &afs4int_int); \
        if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_fstr (pinfo->cinfo, COL_INFO, " :%s:%u", name, afs4int_int); 

#define OUT_AFSRecordLock() \
      offset = dissect_ndr_uint16 (tvb, offset, pinfo, tree, drep, hf_afs4int_l_type, &l_type); \
      offset = dissect_ndr_uint16 (tvb, offset, pinfo, tree, drep, hf_afs4int_l_whence, &l_whence); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_l_start_pos, &l_start_pos); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_l_end_pos, &l_end_pos); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_l_pid, &l_pid); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_l_sysid, &l_sysid); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_l_fstype, &l_fstype); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_l_start_pos_ext, &l_start_pos_ext); \
      offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_l_end_pos_ext, &l_end_pos_ext);

#define OUT_AFSUint2(name) \
        proto_tree_add_uint(tree, name, tvb, offset, sizeof(guint32), tvb_get_ntohl(tvb, offset)); \
        offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep, hf_afs4int_uint, &name); \
        if (check_col (pinfo->cinfo, COL_INFO)) \
        col_append_fstr (pinfo->cinfo, COL_INFO, " :%s:%u", name, afs4int_uint); 

